namespace sap.odm.businesspartner;

using { sap.odm.common.cuid } from '../common/aspects.cds';
using {
  sap.odm.businesspartner.s4.DefaultableCommunicationChannel as S4DefaultableCommunicationChannel,
  sap.odm.businesspartner.s4.Phone as S4Phone,
  sap.odm.businesspartner.s4.Website as S4Website
} from './s4/BusinessPartner.cds';
using {
  sap.odm.common.address.EmailAddress,
  sap.odm.common.address.PhoneNumber
} from '../common/address/CommunicationTypes.cds';
using { sap.odm.common.CountryCode } from '../common/CountryCodes.cds';
using { sap.odm.common.URL } from '../common/types.cds';

/**
 * Abstract entity for communication channel that can be set to
 * default.
 */
aspect DefaultableCommunicationChannel: cuid  {
  /**
   * Defines if this channel is the default.
   */
  isDefault: Boolean default false;
};

/**
 * The default communication channel is email.
 */
@PersonalData.EntitySemantics: 'DataSubjectDetails'
aspect Email: DefaultableCommunicationChannel, S4DefaultableCommunicationChannel  {
  /**
   * The email address. Assumption: to be able to process
   * special characters in the address all connected systems must
   * be unicode enabled.
   */
  @PersonalData.IsPotentiallyPersonal: true
  address: EmailAddress;
};

/**
 * The structured phone number.
 */
aspect StructuredPhoneNumber {
  /**
   * The country code for the phone number.
   */
  country: CountryCode;
  /**
   * The telephone number, which must consist of dialing code and
   * number. The country dialing code can be left out when
   * "country" value is known.
   */
  number: PhoneNumber;
  /**
   * The extension number for the telephone number.
   */
  numberExtension: String(10);
};

/**
 * The default communication channel is phone.
 */
@PersonalData.EntitySemantics: 'DataSubjectDetails'
aspect Phone: DefaultableCommunicationChannel, StructuredPhoneNumber, S4DefaultableCommunicationChannel, S4Phone  {
  /**
   * Indicates if a phone number is for a mobile phone.
   */
  isMobile: Boolean default false;
};

/**
 * The default communication channel is fax.
 */
@PersonalData.EntitySemantics: 'DataSubjectDetails'
aspect Fax: DefaultableCommunicationChannel, StructuredPhoneNumber, S4DefaultableCommunicationChannel  {};

/**
 * The default communication channel is website.
 */
@PersonalData.EntitySemantics: 'DataSubjectDetails'
aspect Website: DefaultableCommunicationChannel, S4Website  {
  /**
   * The URL path for the website.
   */
  @PersonalData.IsPotentiallyPersonal: true
  url: URL;
};
