namespace sap.odm.finance.costobject;

using { sap.odm.orgunit.CompanyCodeId } from '../../orgunit/CompanyCode.cds';
using { sap.odm.finance.ControllingAreaId } from '../types.cds';
using { sap.odm.workforce.WorkforcePerson } from '../../workforce/WorkforcePerson.cds';
using { sap.odm.common.cuid } from '../../common/aspects.cds';

/**
 * Cost center key as defined by SAP S/4HANA Finance. Use
 * company code and cost center ID for legacy posting APIs.
 */
type LocalIdS4: {
  /**
   * Identifier of a specific client across the landscape.
   */
  logicalSystem: String(10);
  /**
   * Company code used in SAP S/4HANA Finance.
   */
  companyCode: CompanyCodeId;
  /**
   * ID of the cost center used in SAP S/4HANA.
   */
  costCenterId: String(10);
  /**
   * ID of the controlling area used in SAP S/4HANA.
   */
  controllingArea: ControllingAreaId;
};

/**
 * Time-dependent attributes for cost centers.
 */
type Attributes: {
  /**
   * Person responsible for the cost center.
   */
  responsible: association to one WorkforcePerson;
  /**
   * Indicates whether the cost center is blocked for primary
   * postings.
   */
  isBlockedForPrimaryPosting: Boolean;
  /**
   * Indicates whether the cost center is blocked for secondary
   * postings.
   */
  isBlockedForSecondaryPosting: Boolean;
  /**
   * Name of the cost center.
   */
  name: localized String(20);
  /**
   * Cost center description to provide background and rationale
   * for the cost center.
   */
  description: localized String(40);
};

/**
 * Cost center is an organizational unit that represents a
 * defined location of cost incurrence.
 */
@ODM.root: true
@odm.doc.alternateTerms: [ 'CostCentre' ]
@odm.doc.relatedGDT: 'CostCentre'
entity CostCenter: cuid  {
  /**
   * Cost center name.
   */
  displayName: String(30);
  /**
   * Identifier of the cost center used in SAP S/4HANA.
   */
  localIdS4: LocalIdS4;
  /**
   * Validity period of the cost center.
   */
  @odm.temporalDate: true
  isValid: Boolean;
  /**
   * Time-dependent attributes of the cost center.
   */
  @odm.temporalDate: true
  attributes: Attributes;
};
