namespace sap.odm.sales.s4;

using { sap.odm.sales.s4.CustomerClassificationCode } from './CustomerClassificationCodes.cds';
using { sap.odm.sales.s4.CustomerAccountGroupCode } from './CustomerAccountGroupCodes.cds';
using { sap.odm.sales.s4.NielsenRegionCode } from './NielsenRegionCodes.cds';
using { sap.odm.sales.s4.DataMediumExchangeIndicatorCode } from './DataMediumExchangeIndicatorCodes.cds';
using { sap.odm.sales.s4.DataExchangeInstructionCode } from './DataExchangeInstructionCodes.cds';
using { sap.odm.sales.s4.CustomerConditionGroupCode } from './CustomerConditionGroupCodes.cds';
using { sap.odm.sales.s4.CfopCategoryCode } from './CfopCategoryCodes.cds';
using { sap.odm.sales.s4.IndustryCode } from './IndustryCodes.cds';
using { sap.odm.sales.s4.AccountTaxTypeCode } from './AccountTaxTypeCodes.cds';

/**
 * Customer related information about a business partner.
 */
aspect CustomerInformation {
  /**
   * Specifies a classification of the customer (for example,
   * classifies the customer as a bulk purchaser).
   */
  customerClassification: CustomerClassificationCode;
  /**
   * If the customer belongs to a group, you can enter a group
   * key here. The group key is freely assignable.
   */
  customerCorporateGroup: String(10);
  /**
   * The account group is a classifying feature within customer
   * master records. It determines number range of the customer
   * account number, whether the number is assigned by the user
   * or by the system and the necessary or possible
   * specifications in the master record.
   */
  customerAccountGroup: CustomerAccountGroupCode;
  /**
   * Indicates that all data in this master record is to be
   * deleted.
   */
  isMarkedForDeletion: Boolean;
  /**
   * Specifies a regional division according to the market
   * categories created by the A. C. Nielsen company.
   */
  nielsenRegion: NielsenRegionCode;
  /**
   * Central sales block for customer.
   */
  salesBlockForCustomer: Boolean;
  /**
   * Indicator to specify Express train station.
   */
  expressTrainStationName: String(25);
  /**
   * Indicator to specify Account number of the master record
   * with the fiscal address.
   */
  fiscalAddress: String(10);
  /**
   * Indicator to specify Business Partner Subject to
   * Equalization Tax.
   */
  isEqualizationTaxSubject: Boolean;
  /**
   * Indicator to specify Business Partner Liable for VAT.
   */
  vatLiability: Boolean;
  /**
   * Indicator to specify Is an alternative payer allowed in
   * document.
   */
  alternativePayerIsAllowed: Boolean;
  /**
   * Indicator to specify Year For Which Sales are Given.
   */
  plannedAnnualSalesYear: String(4);
  /**
   * Indicator to specify Currency of sales figure.
   */
  plannedAnnualSalesCurrency: String(5);
  /**
   * Indicator to specify Annual Sales.
   */
  plannedAnnualSalesAmount: Decimal(15, 2);
  /**
   * Indicator to specify Data Medium Exchange.
   */
  dataMediumExchangeIndicator: DataMediumExchangeIndicatorCode;
  /**
   * Indicator to specify Instruction key for data medium
   * exchange.
   */
  dataExchangeInstruction: DataExchangeInstructionCode;
  /**
   * Indicator to specify Customer condition group 1.
   */
  customerConditionGroup1: CustomerConditionGroupCode;
  /**
   * Indicator to specify Customer condition group 2.
   */
  customerConditionGroup2: CustomerConditionGroupCode;
  /**
   * Indicator to specify Customer condition group 3.
   */
  customerConditionGroup3: CustomerConditionGroupCode;
  /**
   * Indicator to specify Customer condition group 4.
   */
  customerConditionGroup4: CustomerConditionGroupCode;
  /**
   * Indicator to specify Customer condition group 5.
   */
  customerConditionGroup5: CustomerConditionGroupCode;
  /**
   * Indicator to specify CFOP category of Customer.
   */
  cfopCategory: CfopCategoryCode;
  /**
   * International location number (part 1).
   */
  internationalLocationNumber1: String(7);
  /**
   * International location number (part 2).
   */
  internationalLocationNumber2: String(5);
  /**
   * International location number (part 3).
   */
  internationalLocationNumber3: String(1);
  /**
   * Central posting block.
   */
  postingIsBlocked: Boolean;
  /**
   * Company ID of Trading Partner.
   */
  customerTradingPartnerId: String(6);
  /**
   * Industry Code 1.
   */
  industryCode1: IndustryCode;
  /**
   * Industry Code 2.
   */
  industryCode2: IndustryCode;
  /**
   * Industry Code 3.
   */
  industryCode3: IndustryCode;
  /**
   * Industry Code 4.
   */
  industryCode4: IndustryCode;
  /**
   * Industry Code 5.
   */
  industryCode5: IndustryCode;
  /**
   * Fiscal Year Variant.
   */
  fiscalYearVariant: String(2);
  /**
   * Tax Type.
   */
  accountTaxType: AccountTaxTypeCode;
  /**
   * Indicator to specify Customer Extension Code 1.
   */
  customerExtensionCode01: String(2);
  /**
   * Indicator to specify Customer Extension Code 2.
   */
  customerExtensionCode02: String(2);
  /**
   * Indicator to specify Customer Extension Code 3.
   */
  customerExtensionCode03: String(2);
  /**
   * Indicator to specify Customer Extension Code 4.
   */
  customerExtensionCode04: String(2);
  /**
   * Indicator to specify Customer Extension Code 5.
   */
  customerExtensionCode05: String(2);
  /**
   * Indicator to specify Customer Extension Code 6.
   */
  customerExtensionCode06: String(3);
  /**
   * Indicator to specify Customer Extension Code 7.
   */
  customerExtensionCode07: String(3);
  /**
   * Indicator to specify Customer Extension Code 8.
   */
  customerExtensionCode08: String(3);
  /**
   * Indicator to specify Customer Extension Code 9.
   */
  customerExtensionCode09: String(3);
  /**
   * Indicator to specify Customer Extension Code 10.
   */
  customerExtensionCode10: String(3);
  /**
   * Indicator to specify Representative Name.
   */
  representativeName: String(10);
  /**
   * Indicator to specify Business Type.
   */
  businessType: String(30);
  /**
   * Indicator to specify Industry Type.
   */
  industryType: String(30);
};

/**
 * Customer Sales arrangement aspect.
 */
aspect CustomerSalesArrangement {
  /**
   * Human-readable identifier of the sales office.
   */
  salesOfficeDisplayId: String(4);
  /**
   * Human-readable identifier of a sales group.
   */
  salesGroupDisplayId: String(4);
  /**
   * Sales Block for Customer (Sales Area).
   */
  salesBlockForCustomer: Boolean;
  /**
   * Indicates whether a sales order must be delivered completely
   * in a single delivery or whether the order can be partially
   * delivered and completed over a number of deliveries.
   */
  completeDeliveryIsDefined: Boolean;
};

/**
 * Business process related partner function of the customer
 * sales arrangement.
 */
aspect CustomerSalesArrangementFunction {
  /**
   * Customer description of partner (plant, storage location).
   */
  partnerDescription: String(30);
  /**
   * Identifier of the corresponding partner based on Partner
   * type.
   */
  partnerNumber: String(10);
};
